from functools import wraps

from allianceauth.services.hooks import get_extension_logger
from esi.errors import TokenError

from . import __title__
from .utils import LoggerAddTag

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


def fetch_token_for_owner(
    scopes=[
        "esi-universe.read_structures.v1",
        "esi-corporations.read_blueprints.v1",
        "esi-assets.read_corporation_assets.v1",
    ]
):
    """returns valid token for owner.
    Needs to be attached on an Owner method !!

    Args:
    -scopes: Optionally provide the required scopes.
    Otherwise will use all scopes defined for this character.
    """

    def decorator(func):
        @wraps(func)
        def _wrapped_view(owner, *args, **kwargs):
            token, error = owner.token(scopes)
            if error:
                raise TokenError
            logger.debug(
                "%s: Using token %s for `%s`",
                token.character_name,
                token.pk,
                func.__name__,
            )
            return func(owner, token, *args, **kwargs)

        return _wrapped_view

    return decorator



# Generated by Django 3.1.2 on 2020-12-17 05:43

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import blueprints.validators


class Migration(migrations.Migration):

    dependencies = [
        ("eveonline", "0012_index_additions"),
        ("eveuniverse", "0004_effect_longer_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("blueprints", "0002_auto_20201211_1714"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="blueprints",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access this app"),
                    ("request_blueprints", "Can request blueprints"),
                    ("manage_requests", "Can review and accept blueprint requests"),
                    ("add_blueprint_owner", "Can add blueprint owners"),
                    ("view_alliance_blueprints", "Can view alliance's blueprints"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="blueprint",
            name="material_efficiency",
            field=models.PositiveIntegerField(
                help_text="Material efficiency of the blueprint",
                validators=[blueprints.validators.validate_material_efficiency],
            ),
        ),
        migrations.AlterField(
            model_name="blueprint",
            name="time_efficiency",
            field=models.PositiveIntegerField(
                help_text="Time efficiency of the blueprint",
                validators=[blueprints.validators.validate_time_efficiency],
            ),
        ),
        migrations.CreateModel(
            name="Request",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "material_efficiency",
                    models.PositiveIntegerField(
                        help_text="Material efficiency of the blueprint requested",
                        validators=[blueprints.validators.validate_material_efficiency],
                    ),
                ),
                (
                    "time_efficiency",
                    models.PositiveIntegerField(
                        help_text="Time efficiency of the blueprint requested",
                        validators=[blueprints.validators.validate_time_efficiency],
                    ),
                ),
                (
                    "runs",
                    models.PositiveIntegerField(
                        blank=True,
                        help_text="Runs requested or blank for maximum allowed",
                        null=True,
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("OP", "Open"),
                            ("IP", "In Progress"),
                            ("FL", "Fulfilled"),
                            ("CL", "Cancelled"),
                        ],
                        help_text="Status of the blueprint request",
                        max_length=2,
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                (
                    "closed_at",
                    models.DateTimeField(blank=True, db_index=True, null=True),
                ),
                (
                    "eve_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveuniverse.evetype",
                    ),
                ),
                (
                    "fulfulling_user",
                    models.ForeignKey(
                        blank=True,
                        help_text="The user that fulfilled the request, if it has been fulfilled",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "location",
                    models.ForeignKey(
                        help_text="Location of the blueprint requested",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="blueprints.location",
                    ),
                ),
                (
                    "requestee_corporation",
                    models.ForeignKey(
                        help_text="Corporation owning the requested blueprint",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveonline.evecorporationinfo",
                    ),
                ),
                (
                    "requesting_user",
                    models.ForeignKey(
                        help_text="The requesting user",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
    ]



# Generated by Django 3.1.2 on 2020-12-11 17:14

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("blueprints", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="owner",
            name="blueprints_last_error",
        ),
        migrations.RemoveField(
            model_name="owner",
            name="blueprints_last_sync",
        ),
        migrations.AddField(
            model_name="location",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Eve Online location ID of the parent object",
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="blueprints.location",
            ),
        ),
        migrations.AlterField(
            model_name="blueprint",
            name="location_flag",
            field=models.CharField(
                choices=[
                    ("AssetSafety", "AssetSafety"),
                    ("AutoFit", "AutoFit"),
                    ("Bonus", "Bonus"),
                    ("Booster", "Booster"),
                    ("BoosterBay", "BoosterBay"),
                    ("Capsule", "Capsule"),
                    ("Cargo", "Cargo"),
                    ("CorpDeliveries", "CorpDeliveries"),
                    ("CorpSAG1", "CorpSAG1"),
                    ("CorpSAG2", "CorpSAG2"),
                    ("CorpSAG3", "CorpSAG3"),
                    ("CorpSAG4", "CorpSAG4"),
                    ("CorpSAG5", "CorpSAG5"),
                    ("CorpSAG6", "CorpSAG6"),
                    ("CorpSAG7", "CorpSAG7"),
                    ("CrateLoot", "CrateLoot"),
                    ("Deliveries", "Deliveries"),
                    ("DroneBay", "DroneBay"),
                    ("DustBattle", "DustBattle"),
                    ("DustDatabank", "DustDatabank"),
                    ("FighterBay", "FighterBay"),
                    ("FighterTube0", "FighterTube0"),
                    ("FighterTube1", "FighterTube1"),
                    ("FighterTube2", "FighterTube2"),
                    ("FighterTube3", "FighterTube3"),
                    ("FighterTube4", "FighterTube4"),
                    ("FleetHangar", "FleetHangar"),
                    ("FrigateEscapeBay", "FrigateEscapeBay"),
                    ("Hangar", "Hangar"),
                    ("HangarAll", "HangarAll"),
                    ("HiSlot0", "HiSlot0"),
                    ("HiSlot1", "HiSlot1"),
                    ("HiSlot2", "HiSlot2"),
                    ("HiSlot3", "HiSlot3"),
                    ("HiSlot4", "HiSlot4"),
                    ("HiSlot5", "HiSlot5"),
                    ("HiSlot6", "HiSlot6"),
                    ("HiSlot7", "HiSlot7"),
                    ("HiddenModifiers", "HiddenModifiers"),
                    ("Implant", "Implant"),
                    ("Impounded", "Impounded"),
                    ("JunkyardReprocessed", "JunkyardReprocessed"),
                    ("JunkyardTrashed", "JunkyardTrashed"),
                    ("LoSlot0", "LoSlot0"),
                    ("LoSlot1", "LoSlot1"),
                    ("LoSlot2", "LoSlot2"),
                    ("LoSlot3", "LoSlot3"),
                    ("LoSlot4", "LoSlot4"),
                    ("LoSlot5", "LoSlot5"),
                    ("LoSlot6", "LoSlot6"),
                    ("LoSlot7", "LoSlot7"),
                    ("Locked", "Locked"),
                    ("MedSlot0", "MedSlot0"),
                    ("MedSlot1", "MedSlot1"),
                    ("MedSlot2", "MedSlot2"),
                    ("MedSlot3", "MedSlot3"),
                    ("MedSlot4", "MedSlot4"),
                    ("MedSlot5", "MedSlot5"),
                    ("MedSlot6", "MedSlot6"),
                    ("MedSlot7", "MedSlot7"),
                    ("OfficeFolder", "OfficeFolder"),
                    ("Pilot", "Pilot"),
                    ("PlanetSurface", "PlanetSurface"),
                    ("QuafeBay", "QuafeBay"),
                    ("QuantumCoreRoom", "QuantumCoreRoom"),
                    ("Reward", "Reward"),
                    ("RigSlot0", "RigSlot0"),
                    ("RigSlot1", "RigSlot1"),
                    ("RigSlot2", "RigSlot2"),
                    ("RigSlot3", "RigSlot3"),
                    ("RigSlot4", "RigSlot4"),
                    ("RigSlot5", "RigSlot5"),
                    ("RigSlot6", "RigSlot6"),
                    ("RigSlot7", "RigSlot7"),
                    ("SecondaryStorage", "SecondaryStorage"),
                    ("ServiceSlot0", "ServiceSlot0"),
                    ("ServiceSlot1", "ServiceSlot1"),
                    ("ServiceSlot2", "ServiceSlot2"),
                    ("ServiceSlot3", "ServiceSlot3"),
                    ("ServiceSlot4", "ServiceSlot4"),
                    ("ServiceSlot5", "ServiceSlot5"),
                    ("ServiceSlot6", "ServiceSlot6"),
                    ("ServiceSlot7", "ServiceSlot7"),
                    ("ShipHangar", "ShipHangar"),
                    ("ShipOffline", "ShipOffline"),
                    ("Skill", "Skill"),
                    ("SkillInTraining", "SkillInTraining"),
                    ("SpecializedAmmoHold", "SpecializedAmmoHold"),
                    ("SpecializedCommandCenterHold", "SpecializedCommandCenterHold"),
                    ("SpecializedFuelBay", "SpecializedFuelBay"),
                    ("SpecializedGasHold", "SpecializedGasHold"),
                    ("SpecializedIndustrialShipHold", "SpecializedIndustrialShipHold"),
                    ("SpecializedLargeShipHold", "SpecializedLargeShipHold"),
                    ("SpecializedMaterialBay", "SpecializedMaterialBay"),
                    ("SpecializedMediumShipHold", "SpecializedMediumShipHold"),
                    ("SpecializedMineralHold", "SpecializedMineralHold"),
                    ("SpecializedOreHold", "SpecializedOreHold"),
                    (
                        "SpecializedPlanetaryCommoditiesHold",
                        "SpecializedPlanetaryCommoditiesHold",
                    ),
                    ("SpecializedSalvageHold", "SpecializedSalvageHold"),
                    ("SpecializedShipHold", "SpecializedShipHold"),
                    ("SpecializedSmallShipHold", "SpecializedSmallShipHold"),
                    ("StructureActive", "StructureActive"),
                    ("StructureFuel", "StructureFuel"),
                    ("StructureInactive", "StructureInactive"),
                    ("StructureOffline", "StructureOffline"),
                    ("SubSystemBay", "SubSystemBay"),
                    ("SubSystemSlot0", "SubSystemSlot0"),
                    ("SubSystemSlot1", "SubSystemSlot1"),
                    ("SubSystemSlot2", "SubSystemSlot2"),
                    ("SubSystemSlot3", "SubSystemSlot3"),
                    ("SubSystemSlot4", "SubSystemSlot4"),
                    ("SubSystemSlot5", "SubSystemSlot5"),
                    ("SubSystemSlot6", "SubSystemSlot6"),
                    ("SubSystemSlot7", "SubSystemSlot7"),
                    ("Unlocked", "Unlocked"),
                    ("Wallet", "Wallet"),
                    ("Wardrobe", "Wardrobe"),
                ],
                help_text="Additional location information",
                max_length=36,
            ),
        ),
    ]
